#include "config.h"
#include "liblist.h"
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <stdlib.h>
#include "xhelp.h"
#include "pixmap.h"
#include "globals.h"

static lt_dlhandle TryLoading(const char *library)
{
  /* char path[1024]; */
  lt_dlhandle lib = NULL;
  /* try searching in the plugins directory */
  /* strcpy(path,gPluginDir);
     PathAppend(path,library); 
  
     lib = lt_dlopen(path); */
  if (!lib)
  {
    /* try loading normally */
    lib = lt_dlopen(library);
  }
  return lib;
}

static void LibEntryDestroyFunc(LPLIBENTRY pEntry)
{
#ifdef DEBUG
  printf("Destroying module\n");
#endif
  pEntry->pModule->Destroy(pEntry->pModule);
  
  lt_dlclose(pEntry->handle);
}

int LibListAdd(LPLIBLIST pList,LPLIBRARYINIT pInit,const char *library)
{
  /* try to load the library */
  LIBINIT init;
  LPLIBENTRY pEntry;
  lt_dlhandle handle = TryLoading(library);
  if (!handle)
  {
    printf("Unable to load library %s, %s\n",library,lt_dlerror());
    return 0;
  }
  
  if (!(init = (LIBINIT)lt_dlsym(handle,INIT_SYM)))
  {
    printf("Failed to find " INIT_SYM " function in module %s\n",library);
    lt_dlclose(handle);
    return 0;
  }
  /* now do the initialize */
  if (!init(pInit) || pInit->pModule == NULL)
  {
    printf("Module %s failed to initialize\n",library);
    lt_dlclose(handle);
    return 0;
  }
  /* SUCCESSFUL */
  pEntry = (LPLIBENTRY)ListCreateNode(pList,sizeof(LIBENTRY));
  if (!pEntry)
  {
    pInit->pModule->Destroy(pInit->pModule);
    lt_dlclose(handle);
    return 0;
  }

  pEntry->handle = handle;
  pEntry->pInit  = init;
  pEntry->pModule= pInit->pModule;
  
  ListAddTail(pList,(LPNODE)pEntry);
#ifdef DEBUG
  printf("Loaded module %s\n",library);
#endif
  return 1;
}

LPLIBLIST LibListCreate()
{
  return (LPLIBLIST)ListCreate((NODEFREEFUNC)LibEntryDestroyFunc,sizeof(LIST));
}

void LibListDestroy(LPLIBLIST pList)
{
  ListDestroy(pList);
}
